////////////////////////////////////////////////////////////////
//
// Copyright (C) 2005 Affymetrix, Inc.
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License (version 2) as 
// published by the Free Software Foundation.
// 
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// along with this program;if not, write to the 
// 
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
////////////////////////////////////////////////////////////////

#ifndef ROWFILETEST_H
#define ROWFILETEST_H

#include <string>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include "RowFile.h"
#include "Util.h"

using namespace std;
/**
 * @class RowFileTest
 * @brief cppunit class for testing conversion functions.
 */
class RowFileTest : public CppUnit::TestFixture {
  CPPUNIT_TEST_SUITE( RowFileTest );
  CPPUNIT_TEST( testNextRow );
  CPPUNIT_TEST( testLineEnding );
  CPPUNIT_TEST( testGetSet );
  CPPUNIT_TEST( testReadHeader );
  CPPUNIT_TEST_SUITE_END();

public:
  // Test reading the next row.
  void testNextRow();
  // Test checking the type of files
  void testLineEnding();
  // Test our getters and setters
  void testGetSet();
  // Test some of the readers.
  void testReaders();
  // Test how well we read a header.
  void testReadHeader();
};

#endif

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( RowFileTest );

void RowFileTest::testReadHeader() {
  RowFile rf;
  vector<string> lines;
  map<string,vector<string> > header;
  rf.open("input/RowFileTestHeader.txt");
  RowFile::readHeader(rf, header, lines);
  CPPUNIT_ASSERT( header["this"][0] == "that" );
  CPPUNIT_ASSERT( header["this"][1] == "that2" );
  CPPUNIT_ASSERT( header["testing"][0] == "good" );
  CPPUNIT_ASSERT( header["love"][0] == "life" );
  CPPUNIT_ASSERT( header.find("Hello") == header.end() );
}
  

void RowFileTest::testGetSet() {
  RowFile rf;
  rf.setDelim(',');
  CPPUNIT_ASSERT( ',' == rf.getDelim() );
  rf.setComment('*');
  CPPUNIT_ASSERT( '*' == rf.getComment() );
}

void RowFileTest::testLineEnding() {
  RowFile rf, dos, mac, unknown;
  rf.open("input/RowFileTest.1.txt");
  dos.open("input/msdosFormat.txt");
  mac.open("input/macFormat.txt");
  unknown.open("input/noLineEnding.txt");
  CPPUNIT_ASSERT( rf.getFileType() == RowFile::UNIX);
  //Under CYGWIN installed to use DOS line endings, this will look like UNIX
  //CPPUNIT_ASSERT( dos.getFileType() == RowFile::DOS);
  CPPUNIT_ASSERT( mac.getFileType() == RowFile::MAC);
  CPPUNIT_ASSERT( unknown.getFileType() == RowFile::UNKNOWN);
}

void RowFileTest::testReaders() {
  RowFile rf;
  const string *s = NULL;
  vector<const char *> cstrings;
  vector<string> words;
  rf.open("input/RowFileTest.1.txt");
  rf.nextCStringRow(cstrings);
  CPPUNIT_ASSERT( Util::sameString("Hello", cstrings[0]) );
  CPPUNIT_ASSERT( Util::sameString("am", cstrings[3]) );
  s = rf.nextLine();
  CPPUNIT_ASSERT ( *s == "This	row		has	a	null" );
  s = rf.nextRealLine();
  CPPUNIT_ASSERT ( *s == "here	is	the	third	line	of	input" );
}

/** Check and see if we get the next row in our test file.
*/
void RowFileTest::testNextRow() {
  RowFile rf;
  vector<string> words;
  rf.open("input/RowFileTest.1.txt");
  rf.nextRow(words);
  CPPUNIT_ASSERT( words.size() == 4 );
  CPPUNIT_ASSERT( words[1] == "Here" );

  rf.nextRow(words);
  CPPUNIT_ASSERT( words.size() == 6 );
  CPPUNIT_ASSERT( words[1] == "row" );
  CPPUNIT_ASSERT( words[2] == "" );

  rf.nextRow(words);
  CPPUNIT_ASSERT( words.size() == 7 );
  CPPUNIT_ASSERT( words[1] == "is" );

  CPPUNIT_ASSERT(rf.nextRow(words) == false);
  rf.close();
}



