////////////////////////////////////////////////////////////////
//
// Copyright (C) 2005 Affymetrix, Inc.
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License (version 2) as 
// published by the Free Software Foundation.
// 
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// along with this program;if not, write to the 
// 
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
////////////////////////////////////////////////////////////////

#include <TableFile.h>
#include <string>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>

using namespace std;
/**
 * @class TableFileTest
 * @brief cppunit class for testing conversion functions.
 */
class TableFileTest : public CppUnit::TestFixture {
  CPPUNIT_TEST_SUITE( TableFileTest );
  CPPUNIT_TEST( testOpen );
  CPPUNIT_TEST( testWrite );
  CPPUNIT_TEST( testColIndex );
  CPPUNIT_TEST( testRowIndex );
  CPPUNIT_TEST_SUITE_END();

public:
  /** Test reading rows. */
  void testOpen();
  void testWrite();
  void testColIndex();
  void testRowIndex();
  void setUp();
  TableFile tf;
};

void TableFileTest::setUp() {
  tf.setUseRowNames();
  tf.open("input/TableFileTest.txt");
}

void TableFileTest::testColIndex() {
  CPPUNIT_ASSERT( tf.colIndex((char *)"colName1") == 0);
  CPPUNIT_ASSERT( tf.colIndex((char *)"colName2") == 1);
  CPPUNIT_ASSERT( tf.colIndex((char *)"colName3") == 2);
}

void TableFileTest::testRowIndex() {
  CPPUNIT_ASSERT( tf.rowIndex((char *)"row1") == 0);
  CPPUNIT_ASSERT( tf.rowIndex((char *)"row2") == 1);
  CPPUNIT_ASSERT( tf.rowIndex((char *)"blah") == TableFile::npos);
}

/** Check and see if we get the next row in our test file.
*/
void TableFileTest::testOpen() {
  CPPUNIT_ASSERT( tf.numRows() == 2 );
  CPPUNIT_ASSERT( tf.numCols() == 3 );
  CPPUNIT_ASSERT( tf.getData(0,0) == string("rowData1") );
  CPPUNIT_ASSERT( tf.getData(0,1) == string("1") );
  CPPUNIT_ASSERT( tf.getData(1,2) == string("2.0") );
  CPPUNIT_ASSERT( tf.getColName(1) == string("colName2") );
  CPPUNIT_ASSERT( tf.getRowName(1) == string("row2") );
}

void TableFileTest::testWrite() {
  CPPUNIT_ASSERT(tf.write("output/TableFileTest.txt"));
}

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TableFileTest );
