////////////////////////////////////////////////////////////////
//
// Copyright (C) 2005 Affymetrix, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License (version 2) as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program;if not, write to the
//
// Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
////////////////////////////////////////////////////////////////

/**
 * @file   UtilTest.cpp
 * @author Chuck Sugnet
 * @date   Tue Mar 21 10:15:03 PST 2006
 *
 * @brief  Testing the util functions.
 *
 */
#ifndef UTILTEST_H
#define UTILTEST_H

//
#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <iostream>
#include <string>
//
#include "util/Util.h"

using namespace std;
/**
 * @class UtilTest
 * @brief cppunit class for testing conversion functions.
 */
class UtilTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( UtilTest );
  CPPUNIT_TEST( schrageRandomTest );
  CPPUNIT_TEST( stringEndTest );
  CPPUNIT_TEST_SUITE_END();

public:
  /** Test the random number generator. */
  void schrageRandomTest();
  void stringEndTest();
};

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( UtilTest );

void UtilTest::schrageRandomTest() {
  int32_t test = 42;
  int32_t gold[] = {705894,1126542223,1579310009,565444343,807934826,421520601,2095673201,1100194760,1139130650,552121545,229968128,1751246343,1933904666,970724117,526968160,531304892,404315618,694332618,222172928,1733822410,1147638727,1813450982,1582736250,168218361,1157513875,281338952,1852259217,997713207,1013554273,966378307,501383488,26451988,49447387,2133545567,1965890610,1687573175,1225826296,1651931201,1339106791,739215777,806666144,573618697,755349096,1376419055,781211901,113402349,1135284754,338656883,974568031,707121348};
  bool good = true;
  for(int i = 0; i < ArraySize(gold); i++) {
    int32_t result = Util::schrageRandom(&test);
    good &= (gold[i] == result);
  }
  CPPUNIT_ASSERT(good);
}

void UtilTest::stringEndTest()
{
  //
  CPPUNIT_ASSERT(Util::stringEndsWith(""    ,""     )==true );
  CPPUNIT_ASSERT(Util::stringEndsWith(""    ,"a"    )==false);
  CPPUNIT_ASSERT(Util::stringEndsWith("a"   ,"aa"   )==false);
  CPPUNIT_ASSERT(Util::stringEndsWith("aa"  ,"aa"   )==true );
  CPPUNIT_ASSERT(Util::stringEndsWith("aaaa",""     )==true );
  CPPUNIT_ASSERT(Util::stringEndsWith("abb" ,"a"    )==false);
  CPPUNIT_ASSERT(Util::stringEndsWith("abba","a"    )==true );
  CPPUNIT_ASSERT(Util::stringEndsWith("abba","b"    )==false);
  CPPUNIT_ASSERT(Util::stringEndsWith("abba","c"    )==false);
  CPPUNIT_ASSERT(Util::stringEndsWith("f.txt",".txt")==true );
  CPPUNIT_ASSERT(Util::stringEndsWith("f.bin",".txt")==false);
}

#endif /* UTILTEST_H */
