#!/usr/bin/env ruby
puts("Only run snippets of this file. Not all of it.")
exit(1)

require "rubygems"
require "mysql"

SPECIES_DB = "homo_sapiens_core_56_37a" # rattus_norvegicus_core_56_34x mus_musculus_core_56_37i


##### generate transcripthit.tsv for use by exonstruct to generate structDirs
my = Mysql::new("localhost", "mmbgx", "ex0n", SPECIES_DB)
res = my.query("SELECT transcript_stable_id.transcript_id, transcript_stable_id.stable_id, seq_region.name
  FROM transcript_stable_id
    JOIN transcript USING( transcript_id )
    JOIN seq_region USING( seq_region_id )
    JOIN coord_system USING( coord_system_id )
  WHERE coord_system.name = 'chromosome' and attrib = 'default_version' and seq_region.name not like 'HSCHR%' 
  ORDER BY stable_id")

tsv = File.new("transcripthit-#{SPECIES_DB}.tsv", "w")
tsv.puts "#This file was constructed using the #{SPECIES_DB} build of Ensmebl + X:Map"
tsv.puts "#%header0=transcript_id\tstable_id"
tsv.puts "#%header1=\tprobe_id\tgc_count\tprobe_sequence"

r= 0
res.each do |row|
  print "Transcript #{r=r+1}/#{res.num_rows()}\r"
  res2 = my.query "select transcripthit.probe_id, probe.sequence from transcripthit, probe where transcripthit.probe_id=probe.probe_id and transcripthit.transcript_id=#{row[0]}"
  if res2.num_rows() > 0
    tsv.puts "#{row[0]}\t#{row[1]}"
    res2.each do |row2|
      tsv.puts "\t#{row2[0]}\t#{row2[1].count("CG")}\t#{row2[1].tr!('ACTG','TGAC')}"
    end
  end
end
tsv.close


##### generate geneIDs.txt and geneTranscripts.txt for inclusion in structDirs

geneIDs = File.new("geneIDs-#{SPECIES_DB}.txt", "w")
geneTranscripts = File.new "geneTranscripts-#{SPECIES_DB}.txt", "w"

my = Mysql::new("localhost", "mmbgx", "ex0n", SPECIES_DB)
res = my.query("SELECT gene_stable_id.gene_id, stable_id,seq_region.name
  FROM gene_stable_id
    JOIN gene USING( gene_id )
    JOIN seq_region USING( seq_region_id )
    JOIN coord_system USING( coord_system_id )
  WHERE coord_system.name = 'chromosome' and attrib = 'default_version' and seq_region.name not like 'HSCHR%' 
  ORDER BY stable_id")

r=0
res.each do |row|
  print "Gene #{r=r+1}/#{res.num_rows()}\r"
  res2 = my.query "select transcript_stable_id.stable_id from transcript, transcript_stable_id where transcript.transcript_id = transcript_stable_id.transcript_id and transcript.gene_id=#{row[0]}"
  if res2.num_rows() > 0
    res2.each do |t| 
      geneTranscripts.print "#{t[0]}\t"
    end
    geneTranscripts.print "\n"
    geneIDs.puts row[1]
  end
end

print "\n"
geneIDs.close
geneTranscripts.close  


##### generate genehit.tsv for gene-level analysis for use by exonstruct to generate structDirs
##### Note: here we do not take into account multiple hits within same gene (too complicated)
my = Mysql::new("localhost", "mmbgx", "ex0n", SPECIES_DB)
res = my.query("SELECT gene_stable_id.gene_id, stable_id,seq_region.name 
  FROM gene_stable_id
    JOIN gene USING( gene_id )
    JOIN seq_region USING( seq_region_id )
    JOIN coord_system USING( coord_system_id )
  WHERE coord_system.name = 'chromosome' and attrib = 'default_version' and seq_region.name not like 'HSCHR%'
  ORDER BY stable_id")

tsv = File.new("genehit-#{SPECIES_DB}.tsv", "w")
tsv.puts "#This file was constructed using the #{SPECIES_DB} build of Ensmebl + X:Map"
tsv.puts "#Note that this file lists genes. We use the header 'transcript_id' for compatibility with exonstruct"
tsv.puts "#%header0=transcript_id\tstable_id" # it's actually gene_id, but exonstrut looks for "transcript_id"
tsv.puts "#%header1=\tprobe_id\tgc_count\tprobe_sequence"

r= 0
res.each do |row|
  print "Gene #{r=r+1}/#{res.num_rows()}\r"
  res2 = my.query "select distinct transcripthit.probe_id, probe.sequence from transcripthit join probe
  using (probe_id) join transcript using (transcript_id) where
  transcript.gene_id=#{row[0]}"
  if res2.num_rows() > 0
    tsv.puts "#{row[0]}\t#{row[1]}"
    res2.each do |row2|
      tsv.puts "\t#{row2[0]}\t#{row2[1].count("CG")}\t#{row2[1].tr!('ACTG','TGAC')}"
    end
  end
end

tsv.close

print "\n"



######################################
######################################
######################################


### Add miRNAs (can get list of ENSG from Biomart)

my = Mysql::new("localhost", "mmbgx", "ex0n", SPECIES_DB)
tsv = File.new("genehit-#{SPECIES_DB}-miRNA-nodups.tsv", "a") # append

# Open list of miRNA Ensembl gene IDs
miRNAs = []
File.open("miRNAs.txt") do | file |
  while (foo = file.gets)
    miRNAs << foo.chomp
  end
end

miRNAs.each do |mirna|
  res2 = my.query "SELECT transcript.gene_id, hit.probe_id, probe.sequence from probe JOIN hit USING(probe_id) JOIN hit_exon USING( hit_id ) JOIN exon_transcript USING( exon_id ) JOIN transcript USING( transcript_id ) join gene_stable_id using(gene_id) WHERE gene_stable_id.stable_id='#{mirna}'"
  if res2.num_rows() > 0
    tsv.puts "#{res2.fetch_row()[0]}\t#{mirna}\n"
    res2.data_seek(0)
    res2.each do |row2|
      tsv.puts "\t#{row2[1]}\t#{row2[2].count("CG")}\t#{row2[2].tr!('ACTG','TGAC')}\n"
    end
  end
end
tsv.close



### Add custom transcripts (work in progress)
### Note:: geneTranscripts.txt and geneIDs.txt need to be adapted manually after this


tsv = File.new("transcripthit-#{SPECIES_DB}-nodups-FOXP1.tsv", "a") # append

my = Mysql::new("localhost", "mmbgx", "ex0n", SPECIES_DB)
File.open("FOXP1_v56.txt") do | file | 
  while (foo = file.gets) do
    foo = foo.chomp.split(/[[:blank:]]+/)
    tsv.puts "00000000\t#{foo[0]}"
    puts "00000000\t#{foo[0]}"
    numberOfPairs = (foo.size-3)/2
    sqlstr = "seq_region.name=#{foo[1]} and ("
    for i in (0..numberOfPairs-1) do
      sqlstr << "(hit.seq_region_start >= #{foo[i*2 + 3]} and hit.seq_region_end <= #{foo[i*2 + 4]})"
      sqlstr << " or " unless i==numberOfPairs-1
    end
    sqlstr << ") and hit.seq_region_strand=#{foo[2]}"
    res = my.query "select probe_id, hit.seq_region_start, hit.seq_region_end from hit join seq_region using (seq_region_id) join coord_system using (coord_system_id) where  coord_system.name='chromosome' and attrib = 'default_version' and seq_region.name not like 'HSCHR%' and #{sqlstr} order by hit.seq_region_start;"
    res.each do |row|
      res2 = my.query "select probe.sequence from probe where probe_id=#{row[0]}"
      r = res2.fetch_row
      tsv.puts "\t#{row[0]}\t#{r[0].count("CG")}\t#{r[0].tr!('ACTG','TGAC')}\n"
      puts "\t#{row[0]}\t#{r[0].count("CG")}\t#{r[0].tr!('ACTG','TGAC')}\n"
    end
  end
end

tsv.close
