/*
 *  This file is part of BGX, the Bayesian Gene eXpression program.
 *  Copyright (c) 2008 Ernest Turro <ernest.turro@ic.ac.uk>
 *
 *  BGX is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  BGX is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

// Add a vector to specified columns in a matrix
#include "loessave.hh"
#include <stdio.h>
#include <stdlib.h>

/* reverse:  reverse string s in place */
void reverse(char s[])
{
    int i, j;
    char c;

    for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}
/* itoa:  convert n to characters in s */
void itoa(int n, char s[])
{
    int i, sign;

    if ((sign = n) < 0)  /* record sign */
        n = -n;          /* make n positive */
    i = 0;
    do {       /* generate digits in reverse order */
        s[i++] = n % 10 + '0';   /* get next digit */
    } while ((n /= 10) > 0);     /* delete it */
    if (sign < 0)
        s[i++] = '-';
    s[i] = '\0';
    reverse(s);
} 


extern "C" {
  SEXP loessave(SEXP pathToMus, SEXP nArrays, SEXP sampleSets, SEXP nConds, SEXP loessMatrix,
              SEXP nMus, SEXP nIters, SEXP outDir) {

    const char *poutDir = CHAR(STRING_ELT(outDir, 0));
    int *pnArrays=INTEGER(nArrays);
    char *ppathToMus[*pnArrays];
    for(int i=0; i < *pnArrays; i++) {
      ppathToMus[i] = R_alloc(strlen(CHAR(STRING_ELT(pathToMus, i))), sizeof(char));
      strcpy(ppathToMus[i], CHAR(STRING_ELT(pathToMus, i)));
    }

    int *psampleSets=INTEGER(sampleSets);
    int *pnConds=INTEGER(nConds);
    double *ploessMatrix=REAL(loessMatrix);
    int *pnMus=INTEGER(nMus);
    int *pnIters=INTEGER(nIters);

    int arrayoffset=0;
    for(int set=0; set < *pnConds; set++) {
      double *curmu = new double[*pnIters * *pnMus];
      double temp;
      for(int r=0; r < psampleSets[set]; r++) {
        Rprintf(".");
        FILE *f = fopen(ppathToMus[arrayoffset+r], "r");
        if(f==NULL) {
          Rprintf("Error:82: can't access %s.\n", ppathToMus[arrayoffset+r]);
          return(NULL_USER_OBJECT);
        }
        for(int i=0; i < *pnIters; i++) {
          for(int j=0; j < *pnMus; j++) {
            fscanf(f, "%lf", &temp);
            curmu[i**pnMus + j] += temp - ploessMatrix[(arrayoffset+r) * *pnMus + j];
          }
        }
        fclose(f);
      }
      char *outmu = (char *)calloc(strlen(poutDir) + 8, sizeof(char));
      strcat(outmu, poutDir);
      strcat(outmu, "/mu.");
      char c[3];
      itoa(set+1, c);
      strcat(outmu, c);
      FILE *f = fopen(outmu, "w");
      if(f==NULL) {
        Rprintf("Error:101: can't access %s.\n", outmu);
        return(NULL_USER_OBJECT);
      }

      for(int i=0; i < *pnIters; i++) {
        for(int j=0; j < *pnMus; j++) {
          fprintf(f, "%lf ", curmu[i**pnMus + j]/(double)psampleSets[set]);
        }
        fprintf(f,"\n");
      }
      fclose(f);
      arrayoffset += psampleSets[set];
      delete curmu;
    }
    return(NULL_USER_OBJECT);
  }
}
